<?php 
namespace WHMCS\Route;


class UriPath implements Contracts\MapInterface
{
    use HandleMapTrait;

    private $baseUri = "";
    protected $deferredRoutes = array(  );

    const MODE_REWRITE = "rewrite";
    const MODE_ACCEPTPATHINFO = "acceptpathinfo";
    const MODE_BASIC = "basic";
    const MODE_UNKNOWN = "unknown";
    const SETTING_MODE = "RouteUriPathMode";

    public function __construct($baseUri = "")
    {
        if( empty($baseUri) ) 
        {
            $this->baseUri = \WHMCS\Utility\Environment\WebHelper::getBaseUrl();
        }
        else
        {
            $this->baseUri = $baseUri;
        }

    }

    public function mapRoute($route)
    {
        $attributeName = $this->getMappedAttributeName();
        if( !isset($route["name"]) || !isset($route[$attributeName]) ) 
        {
            return $this;
        }

        $this->routes[$route["name"]] = $route[$attributeName];
        return $this;
    }

    public function getMappedAttributeName()
    {
        return "canonicalPath";
    }

    public function getSafeNonRoutablePath()
    {
        $uri = new \Zend\Diactoros\Uri();
        $routePath = "/route-not-defined";
        $base = $this->baseUri;
        if( substr($base, -1) == "/" ) 
        {
            $base = substr($base, -1);
        }

        $mode = $this->getMode();
        if( $mode == self::MODE_REWRITE ) 
        {
            $uri = $uri->withPath($base . $routePath);
        }
        else
        {
            if( $mode == self::MODE_ACCEPTPATHINFO ) 
            {
                $uri = $uri->withPath($base . "/index.php" . $routePath);
            }
            else
            {
                $uri = $uri->withPath($base . "/index.php")->withQuery("rp=" . $routePath);
            }

        }

        return $uri;
    }

    public function getRawPath($routeName, $routeVars)
    {
        $routePath = $this->buildRoutePath($routeName, $routeVars);
        if( substr($routePath, 0, 1) != "/" ) 
        {
            $routePath = "/" . $routePath;
        }

        return $routePath;
    }

    public function getPath($routeName, ...$routeVars)
    {
        $routePath = $this->getRawPath($routeName, $routeVars);
        $mode = $this->getMode();
        $base = $this->baseUri;
        if( substr($base, -1) == "/" ) 
        {
            $base = substr($base, -1);
        }

        if( defined("ADMINAREA") ) 
        {
            $adminBase = \WHMCS\Admin\AdminServiceProvider::getAdminRouteBase();
            if( strpos($routePath, $adminBase) === 0 && $mode !== self::MODE_REWRITE && substr($base, -1 * strlen($adminBase)) != $adminBase ) 
            {
                $base .= $adminBase;
            }

        }

        $uri = new \Zend\Diactoros\Uri();
        if( $mode == self::MODE_REWRITE ) 
        {
            $uri = $uri->withPath($base . $routePath);
        }
        else
        {
            if( $mode == self::MODE_ACCEPTPATHINFO ) 
            {
                $uri = $uri->withPath($base . "/index.php" . $routePath);
            }
            else
            {
                $uri = $uri->withPath($base . "/index.php")->withQuery("rp=" . $routePath);
            }

        }

        return $uri;
    }

    public function getRouteCollector()
    {
        return \DI::make("Route\\RouteCollector");
    }

    protected function getDeferredRoutePath($routeName)
    {
        $route = null;
        try
        {
            $matches = array(  );
            foreach( $this->routes as $key => $value ) 
            {
                if( strpos($routeName, $key) === 0 ) 
                {
                    $matches[$key] = $value;
                }

            }
            if( empty($matches) ) 
            {
                return $route;
            }

            $collector = $this->getRouteCollector();
            foreach( $matches as $key => $value ) 
            {
                if( $value instanceof Contracts\DeferredProviderInterface ) 
                {
                    unset($this->routes[$key]);
                    $value->registerRoutes($collector);
                    $route = $this->getMappedRoute($routeName);
                    if( !is_null($route) ) 
                    {
                        break;
                    }

                }

            }
        }
        catch( \Exception $e ) 
        {
            logActivity("Failed to expand route " . $routeName);
        }
        return $route;
    }

    protected function buildRoutePath($routeName, $routeVars)
    {
        $requireVariablePattern = "#{[^}]*}#";
        $optionalPathPattern = "#(\\[(.*)\\])#";
        $routePath = $this->getMappedRoute($routeName);
        if( !$routePath ) 
        {
            $routePath = $this->getDeferredRoutePath($routeName);
        }

        if( is_null($routePath) ) 
        {
            throw new \RuntimeException("Invalid route link name: " . $routeName);
        }

        $routePathParts = explode("[", $routePath, 2);
        if( preg_match_all($requireVariablePattern, $routePathParts[0], $requiredInterpolations) ) 
        {
            $requiredCount = count($requiredInterpolations[0]);
            $providedCount = count($routeVars);
            if( $providedCount < $requiredCount ) 
            {
                throw new \RuntimeException("URI route path '" . $routeName . "' requires " . $requiredCount . " " . "interpolation variables, " . $providedCount . " provided.");
            }

        }

        $tmpRouteVars = $routeVars;
        $interpolatePlaceHolder = function() use (&$tmpRouteVars)
{
    if( count($tmpRouteVars) ) 
    {
        $value = array_shift($tmpRouteVars);
        if( $value && !is_numeric($value) ) 
        {
            return getModRewriteFriendlyString($value);
        }

        return (string) $value;
    }

    return "";
}

;
        $link = preg_replace_callback($requireVariablePattern, $interpolatePlaceHolder, $routePath);
        if( preg_match($optionalPathPattern, $routePath, $matches) ) 
        {
            if( preg_match($requireVariablePattern, $matches[2]) ) 
            {
                $tmpRouteVars = array(  );
                $emptyInterpolatedOption = preg_replace_callback($requireVariablePattern, $interpolatePlaceHolder, $matches[2]);
                $invalidOption = "[" . $emptyInterpolatedOption . "]";
                preg_match($optionalPathPattern, $link, $interpolatedMatches);
                if( $interpolatedMatches[1] == $invalidOption ) 
                {
                    $link = preg_replace($optionalPathPattern, "", $link);
                }
                else
                {
                    $link = preg_replace($optionalPathPattern, "\\2", $link);
                }

            }
            else
            {
                $link = str_replace($matches[0], "", $link);
            }

        }

        return $link;
    }

    public static function getAllKnownModes()
    {
        return array( static::MODE_REWRITE, static::MODE_ACCEPTPATHINFO, static::MODE_BASIC );
    }

    public function setMode($mode)
    {
        if( $mode && !in_array($mode, static::getAllKnownModes()) ) 
        {
            throw new \RuntimeException(sprintf("Invalid \"mode\" value \"%s\"", $mode));
        }

        \WHMCS\Config\Setting::setValue(static::SETTING_MODE, $mode);
    }

    public function getMode()
    {
        $mode = \WHMCS\Config\Setting::getValue(static::SETTING_MODE);
        if( in_array($mode, self::getAllKnownModes()) ) 
        {
            return $mode;
        }

        return self::MODE_UNKNOWN;
    }

}


